import inspect

from django.test import TestCase

from projector import settings
from projector.models import Project
from projector.forms import ProjectCreateForm
from projector.forms import ProjectMembershipPermissionsForm
from projector.forms import ProjectTeamPermissionsForm
from projector.forks import BaseExternalForkForm
from projector.utils.basic import str2obj
from projector.views.users import can_fork_external

from guardian.shortcuts import get_perms_for_model

class SettingsTest(TestCase):

    def setUp(self):
        """
        Remember settings at the beginning of the test.
        """
        members = inspect.getmembers(settings)
        self._settings = dict(((key, val) for key, val in members
            if key.isupper()))

    def tearDown(self):
        """
        Reset settings to default values.
        """
        for key, val in self._settings.items():
            setattr(settings, key, val)

    def test_PRIVATE_ONLY(self):
        settings.PRIVATE_ONLY = False
        form = ProjectCreateForm()
        self.assertEqual(len(form.fields['public'].choices), 2)

        settings.PRIVATE_ONLY = True
        form = ProjectCreateForm()
        self.assertEqual(len(form.fields['public'].choices), 1)
        self.assertEqual(form.fields['public'].choices[0][0], u'private')

    def test_EDITABLE_PERMISSIONS(self):
        all_perms = set([p.codename for p in get_perms_for_model(Project)])
        editable_perms = set(settings.get_config_value('EDITABLE_PERMISSIONS'))
        self.assertTrue(editable_perms.issubset(all_perms))
        self.assertTrue('add_project' not in editable_perms,
            "EDITABLE_PERMISSIONS cannot contain add_project permission as "
            "it is global, not object-specific")

        form = ProjectMembershipPermissionsForm()
        form_perms = set((c[0] for c in form.fields['permissions'].choices))
        self.assertTrue(form_perms == editable_perms)

        form = ProjectTeamPermissionsForm()
        form_perms = set((c[0] for c in form.fields['permissions'].choices))
        self.assertTrue(form_perms == editable_perms)

    def test_FORK_FORMS(self):
        map = settings.FORK_EXTERNAL_MAP.items()
        for key, val in map:
            ForkForm = str2obj(val)
            self.assertTrue(issubclass(ForkForm, BaseExternalForkForm))

    def test_FORK_EXTERNAL_both_on(self):
        settings.FORK_EXTERNAL_ENABLED = True
        settings.FORK_EXTERNAL_MAP = {'not': 'empty'}
        self.assertTrue(can_fork_external())

    def test_FORK_EXTERNAL_both_off(self):
        settings.FORK_EXTERNAL_ENABLED = False
        settings.FORK_EXTERNAL_MAP = {}
        self.assertFalse(can_fork_external())

    def test_FORK_EXTERNAL_mix(self):
        settings.FORK_EXTERNAL_MAP = True
        settings.FORK_EXTERNAL_MAP = {}
        self.assertFalse(can_fork_external())

        settings.FORK_EXTERNAL_ENABLED = False
        settings.FORK_EXTERNAL_MAP = {'not': 'empty'}
        self.assertFalse(can_fork_external())




from django.test import TestCase

from projector.utils.email import extract_emails


class ExtractEmailsTest(TestCase):

    def test_extract_single(self):

        def test_email(email, text):
            emails = extract_emails(text)
            self.assertEqual(len(emails), 1, '\n'.join(('',
                "Tested email: %s" % email,
                "Tested text: %s" % text,
                "We should get exactly one email address from the given text")))
            self.assertEqual(emails[0], email, '\n'.join(('',
                "Tested email: %s" % email,
                "Tested text: %s" % text,
                "We should get exactly one email address from the given text")))

        pairs = (
            ('jodoe@example.com', 'jodoe@example.com'),
            ('jodoe@example.com', '<jodoe@example.com>'),
            ('j.doe@example.com', 'j.doe@example.com'),
            ('j.doe@example.com', 'J. Doe j.doe@example.com'),
            ('j.doe@example.com', 'J. Doe <j.doe@example.com>'),
            ('j.doe@example.com', 'J.Doe<j.doe@example.com>'),
            ('j.doe@example.com', 'J.Doe\n<j.doe@example.com>'),
            ('j.doe@example.com', 'foobar\t<j.doe@example.com>'),
            ('j.doe@example.com', 'foobar <j.doe@example.com>'),
            ('j.doe@example.com', 'foobar j.doe@example.com\nbaz'),
            ('j.doe@example.com', 'JDoe j.doe@example.com'),
            ('j.doe@example.com', 'j.doe@example.com J. Doe'),
        )
        for email, text in pairs:
            test_email(email, text)

    def test_extract_multiple(self):

        text = ("my email is lukaszbalcerzak@gmail.com and his: "
                "j.doe@example.com")

        self.assertEqual(
            set(('lukaszbalcerzak@gmail.com', 'j.doe@example.com')),
            set((extract_emails(text)))
        )




import logging

from django.db.models.signals import post_save, post_delete
from django.contrib.auth.models import User
from django.contrib.auth.tokens import default_token_generator
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from projector.settings import get_config_value
from projector.models import Project, Task, WatchedItem
from projector.signals import post_fork, setup_project
from projector.tasks import setup_project as setup_project_task

from richtemplates.utils import get_user_profile_model

from vcs.web.simplevcs.signals import retrieve_hg_post_push_messages

def request_new_profile(sender, instance, **kwargs):
    """
    Creation of profile for new users
    """
    _UserProfile = get_user_profile_model()
    if _UserProfile is None:
        return
    profile, created = _UserProfile.objects.get_or_create(
        user=instance,
    )
    if created is True:
        logging.debug("Creating profile for %s ..." % instance)
        profile.activation_token = default_token_generator.make_token(instance)
        profile.save()
        logging.debug("Created profile's id: %s" % profile.id)

def setup_project_listener(sender, instance, vcs_alias=None,
        workflow=None, **kwargs):
    """
    Task ``setup_project`` for signals framework.
    """
    if get_config_value('CREATE_PROJECT_ASYNCHRONOUSLY'):
        func = setup_project_task.delay
    else:
        func = setup_project_task
    logging.info("Calling setup_project task for instance %s" % instance)
    return func(instance, vcs_alias, workflow)


def fork_done(sender, fork, **kwargs):
    """
    Action made after project is forked.
    """
    pass

def task_save_listener(sender, instance, **kwargs):
    """
    Action made after task is saved (created or updated).
    """
    if kwargs['created'] is True:
        # Task was created
        pass
    else:
        # Task was updated
        pass

def watcheditem_save_listener(sender, instance, **kwargs):
    if kwargs['created'] is True:
        logging.info("%s started watching %s" % (instance.user,
            instance.content_object))
    else:
        logging.info("%s already watching %s" % (instance.user,
            instance.content_object))

def watcheditem_delete_listener(sender, instance, **kwargs):
    logging.info("%s stopped watching %s" % (instance.user,
        instance.content_object))


def hg_extra_messages(sender, repository, **kwargs):
    """
    Adds extra messages appended to request after successful push
    to mercurial repository.
    """
    size = repository.info.size
    msg = _('Repository size: %s' % filesizeformat(size))
    sender.messages.append(msg)


def start_listening():
    """
    As listeners use projectors' models we need to connect signals after they
    are registered so it is implemented as function called at the end of models
    module.
    """
    post_save.connect(request_new_profile, sender=User)
    post_save.connect(task_save_listener, sender=Task)
    post_save.connect(watcheditem_save_listener, sender=WatchedItem)
    post_delete.connect(watcheditem_delete_listener, sender=WatchedItem)

    # Projector signals connection
    post_fork.connect(fork_done)
    setup_project.connect(setup_project_listener, sender=Project)
    retrieve_hg_post_push_messages.connect(hg_extra_messages,
        sender=None)




import logging

from django.contrib.auth.decorators import login_required, permission_required
from django.views.generic import list_detail, create_update

from projector.models import ProjectCategory

@login_required
def project_category_detail(request, project_category_slug,
        template_name='projector/project_category/detail.html'):
    logging.debug("project_category_detail called")
    kwargs = {
        'queryset' : ProjectCategory.objects.all(),
        'slug' : project_category_slug,
        'template_name' : template_name,
        'template_object_name' : 'project_category',
        'extra_context' : {}
    }

    return list_detail.object_detail(request, **kwargs)

@login_required
@permission_required('projector.add_project_category')
def project_category_create(request, template_name='projector/project_category/create.html'):
    logging.debug("project_category_create called")
    kwargs = {
        'model' : ProjectCategory,
        'template_name' : template_name,
    }

    return create_update.create_object(request, **kwargs)

def project_category_list(request, template_name='projector/project_category/list.html'):
    logging.debug("project_category_list called")
    kwargs = {
        'queryset' : ProjectCategory.objects.all(),
        'template_name' : template_name,
        'template_object_name' : 'project_category',
    }

    return list_detail.object_list(request, **kwargs)




from django.contrib.syndication.feeds import Feed

from projector.models import Project

class LatestProjectsFeed(Feed):
    title = "Latest projects"
    link = "/projects/"
    description = "Updates on additions to projects."

    def items(self):
        return Project.objects\
            .order_by('-created_at')\
            .filter(public=True)\
            [:5] # Limiting query





from version import *
from test_component import *
from test_concurrency import *
from test_controllers import *
from test_emails import *
from test_fork import *
from test_helpers import *
from test_milestone import *
from test_permissions import *
from test_settings import *
from test_statustransition import *
from test_status import *
from test_teams import *
from test_templatetags import *
from test_user2team_conversion import *
from test_utils import *



from django.db.models import Count
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext as _
from django.contrib import messages

from projector.models import Component
from projector.views.project import ProjectView
from projector.forms import ComponentForm

class ComponentListView(ProjectView):
    """
    Returns components view.
    """

    template_name = 'projector/project/components/home.html'

    def response(self, request, username, project_slug):
        component_list = self.project.component_set\
            .annotate(Count('task'))
        context = {
            'project': self.project,
            'component_list': component_list,
        }

        return context

class ComponentDetailView(ProjectView):
    """
    Returns component detail view.
    """

    template_name = 'projector/project/components/detail.html'

    def response(self, request, username, project_slug, component_slug):
        component = get_object_or_404(Component, project=self.project,
            slug=component_slug)
        context = {
            'project': self.project,
            'component': component,
        }
        return context

class ComponentCreateView(ProjectView):
    """
    Adds component for project.
    """

    template_name = 'projector/project/components/add.html'
    perms = ['change_project']

    def response(self, request, username, project_slug):
        component = Component(project=self.project)
        form = ComponentForm(request.POST or None, instance=component)
        if request.method == 'POST' and form.is_valid():
            component = form.save()
            msg = _("Component added successfully")
            messages.success(request, msg)
            return redirect(component.get_absolute_url())
        context = {
            'project': self.project,
            'form': form,
        }
        return context

class ComponentEditView(ProjectView):
    """
    Edits chosen component.
    """

    template_name = 'projector/project/components/edit.html'
    perms = ['change_project']

    def response(self, request, username, project_slug, component_slug):
        component = get_object_or_404(Component,
            project=self.project, slug=component_slug)
        form = ComponentForm(request.POST or None, instance=component)
        if request.method == 'POST' and form.is_valid():
            component = form.save()
            msg = _("Component updated successfully")
            messages.success(request, msg)
            return redirect(component.get_absolute_url())
        context = {
            'project': self.project,
            'form': form,
        }
        return context




from django.test import TestCase

from projector.utils.helpers import Choices

class ChoicesTest(TestCase):

    def test(self):

        class Status(Choices):
            FIRST = 1
            SECOND = 2
            THIRD = 3

        choices = Status.as_choices()

        self.assertEqual(choices,
            [
                (1, u'First'),
                (2, u'Second'),
                (3, u'Third'),
            ]
        )

    def test_with_underscore(self):

        class Status(Choices):
            FIRST = 1
            SECOND = 2
            THIRD_AND_LAST = 3

        choices = Status.as_choices()

        self.assertEqual(choices,
            [
                (1, u'First'),
                (2, u'Second'),
                (3, u'Third and last'),
            ]
        )

    def test_with_not_all_uppered(self):

        class Status(Choices):
            FIRST = 1
            SECOND = 2
            third = 3

        choices = Status.as_choices()

        self.assertEqual(choices,
            [
                (1, u'First'),
                (2, u'Second'),
            ]
        )

    def test_with_no_integers(self):

        class Status(Choices):
            FIRST = 1
            SECOND = 2
            THIRD = 'third'

        choices = Status.as_choices()

        self.assertEqual(choices,
            [
                (1, u'First'),
                (2, u'Second'),
            ]
        )

    def test_order(self):

        class Status(Choices):
            SECOND = 2
            NEXT = 900
            THIRD = 3
            ERROR = -1
            FIRST = 1

        choices = Status.as_choices()
        ints = [c[0] for c in choices]
        self.assertEqual(ints, [-1, 1, 2, 3, 900])




import re

EMAIL_RE = re.compile(r'[A-Z0-9+_\-\.]+@[0-9A-Z][.-0-9A-Z]*\.[A-Z]{2,6}',
    re.IGNORECASE)

def extract_emails(text):
    """
    Returns list of emails founded in a given ``text``.
    """
    emails = re.findall(EMAIL_RE, text)
    return emails




from django.conf.urls.defaults import *
from django.conf import settings
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    (r'^$', 'projector.views.users.UserHomepageView'),

    (r'^accounts/', include('registration.urls')),
    (r'^projector/', include('projector.urls')),

    (r'^admin_tools/', include('admin_tools.urls')),
    (r'^admin/', include(admin.site.urls)),
)

if settings.DEBUG:
    urlpatterns += patterns('',
        (r'^%s/(?P<path>.*)$' % settings.MEDIA_URL.strip('/'),
            'django.views.static.serve',
            {'document_root': settings.MEDIA_ROOT}),
)

if 'rosetta' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^rosetta/', include('rosetta.urls')),
    )




def get_homedir(project):
    """
    Returns unique home directory path for each project. Returned path should be
    relative to :setting:`PROJECTOR_PROJECTS_ROOT_DIR`.

    :param project: instance of :py:class:`projector.models.Project`

    .. note ::
       This is simplest possible implementation but on the other hand returned
       value depends solely on the primary key of the given ``project``. It is
       important **not** to use other fields as they may change. Primary key
       should never be changed while even creation date (``created_at`` field)
       may be edited.

       For tests however, this is not acceptable as test runner
       rollbacks queries which should normally be committed. Rollbacks make ID
       value not stable - as we run more and more tests we always got projects
       with same id (starting with 1 for each test). That said, test runner
       must use other *project homedir generator*.

    Implementation::

        return str(project.pk)

    """
    return str(project.pk)


class Choices(object):
    """
    Inspired by same object from Repocracy.
    """

    @classmethod
    def as_choices(cls):
        from django.utils.translation import ugettext as _
        keys = [key for key in dir(cls) if isinstance(getattr(cls, key), int)\
            and key.isupper()]
        slugs = dict((key, key.capitalize().replace('_', ' ')) for key in keys)
        choices = [(getattr(cls, key), _(slugs[key])) for key in keys]
        choices.sort()
        return choices

    @classmethod
    def as_dict(cls):
        return dict(cls.as_choices())

    @classmethod
    def as_json_choices(cls):
        choices = cls.as_dict()
        from django.utils.simplejson import dumps
        return dumps(choices)




import re

from django.conf import settings
from django.utils.encoding import smart_str, force_unicode
from django.utils.safestring import mark_safe
from django.utils.html import escape
from django import template
from django.utils.translation import ugettext as _

from projector.models import Task
from projector.models import get_user_from_string
from projector.settings import get_config_value
from projector.utils.email import EMAIL_RE

register = template.Library()

def restructuredtext(value):
    try:
        from docutils.core import publish_parts
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError("Error in {% restructuredtext %}"
                "filter: The Python docutils library isn't installed.")
        return force_unicode(value)
    else:
        docutils_settings = getattr(settings,
            "RESTRUCTUREDTEXT_FILTER_SETTINGS", {})
        parts = publish_parts(source=smart_str(value), writer_name="html4css1",
            settings_overrides=docutils_settings)
        return mark_safe(force_unicode(parts["fragment"]))

@register.filter
def hide_email(value, sub=None):
    """
    Hide all emails founded within given value and replace them with given
    ``sub``stitution. If no ``sub`` is given,
    :setting:`PROJECTOR_HIDDEN_EMAIL_SUBSTITUTION` would be used.
    """
    if sub is None:
        sub = get_config_value('HIDDEN_EMAIL_SUBSTITUTION')
    value = EMAIL_RE.sub(sub, value)
    return value

@register.filter
def changeset_message(value, project=None, path=None):
    if (project is None and path is None) or (project and path):
        raise template.TemplateSyntaxError, "Using changeset_message filter "\
            "without (or with both) ``project`` and ``path`` is not allowed. "\
            "You have to exactly one of them"
    value = escape(value)
    if project:
        def repl(m):
            id = int(m.group(1))
            try:
                task = project.get_task(id)
                message = _("Summary") + ": %s" % task.summary + "\n" + \
                    _("Status") + ": %s" % task.status
                archon = '<a href="%s" class="show-tipsy" title="%s">#%d</a>'\
                    % (task.get_absolute_url(), escape(message), id)
                if task.status.is_resolved:
                    archon = '<strike>' + archon + '</strike>'
            except Task.DoesNotExist:
                notask_message = _("There is no such task for this project")
                archon = '<span class="show-tipsy" title="%s">#%d</span>'\
                    % (notask_message, id)
            return archon
        #import pdb; pdb.set_trace()
        value = re.sub(r'#(\d+)', repl, value)
    if path:
        raise NotImplementedError
    return mark_safe(value)

class FetchUserNode(template.Node):
    def __init__(self, value, context_var, get_from_context):
        self.value = value
        self.context_var = context_var
        self.get_from_context = get_from_context

    def render(self, context):
        if self.get_from_context:
            try:
                var = template.Variable(self.value).resolve(context)
            except template.VariableDoesNotExist:
                raise template.TemplateSyntaxError("Cannot resolve %s from "
                    "context" % self.value)
            user = get_user_from_string(var)
        else:
            user = get_user_from_string(self.value)
        context[self.context_var] = user
        return ''

@register.tag(name='fetch_user')
def do_fetch_user(parser, token):
    """
    Parses ``fetch_user`` tag which should be in format:
    {% fetch_user RAWID as "context_var" %}
    """
    format = '{% fetch_user RAWID as "context_var" %}'
    bits = token.split_contents()
    error_msg = "fetch_user tag should be in format: %s" % format
    if len(bits) == 4:
        if bits[2] != 'as':
            raise template.TemplateSyntaxError(error_msg)
        if bits[3][0] not in ['"', "'"] or bits[3][0] != bits[3][-1]:
            raise template.TemplateSyntaxError(error_msg)
        context_var = bits[3][1:-1]
        value = bits[1]
        if value[0] in '"\'' and value[0] == value[-1]:
            value = value.strip('"\'')
            get_from_context = False
        else:
            get_from_context = True
        return FetchUserNode(value, context_var, get_from_context)
    raise template.TemplateSyntaxError(error_msg)





class TestWorkflow(object):

    components = (
        {
            'name': u'global',
        },
    )

    task_types = (
        {
            'name': u'task',
            'order': 1,
        },
    )

    priorities = (
        {
            'name': u'minor',
            'order': 1,
        },
    )

    statuses = (
        {
            'name': u'new',
            'order': 1,
            'is_resolved': False,
            'is_initial': True,
        },
    )




from django.contrib import messages
from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django.shortcuts import redirect

from projector.views.project import ProjectView
from projector.utils.lazy import LazyProperty

from vcs.web.simplevcs.views import browse_repository, diff_file, diff_changeset


class RepositoryView(ProjectView):

    perms_private = ['view_project', 'can_read_repository']

    @LazyProperty
    def has_errors(self):
        """
        Default error handling for repository-related views. See also
        ``get_error_response``.
        """
        response = None
        if not self.project.repository_id:
            messages.info(self.request, _("Project has no repository"))
            response = redirect(self.project)
        if not self.project._get_repo_path():
            msg = _("There is something wrong with project's repository")
            messages.error(self.request, msg)
            response = redirect(self.project)
        if not self.project.repository or not self.project.repository.revisions:
            messages.info(self.request, _("Repository has no changesets yet"))
            response = redirect(self.project)
        return response

    def get_error_response(self):
        """
        Combined with ``has_errors`` could be used like this::

            def response(self, request):
                if self.has_errors:
                    return self.get_error_response()
                ...

        """
        if self.has_errors:
            return self.has_errors
        return


class RepositoryBrowse(RepositoryView):

    template_name = 'projector/project/repository/browse.html'

    def response(self, request, username, project_slug, rel_repo_url='',
            revision='tip'):
        if self.has_errors:
            return self.get_error_response()
        repo_info = {
            'repository': self.project.repository,
            'revision': revision,
            'node_path': rel_repo_url,
            'template_name': self.template_name,
            'extra_context': {
                'project': self.project,
            },
        }
        return browse_repository(self.request, **repo_info)


class RepositoryFileDiff(RepositoryView):

    template_name = 'projector/project/repository/diff.html'

    def response(self, request, username, project_slug, revision_old,
            revision_new, rel_repo_url):
        if self.has_errors:
            return self.get_error_response()
        diff_info = {
            'repository': self.project.repository,
            'revision_old': revision_old,
            'revision_new': revision_new,
            'file_path': rel_repo_url,
            'template_name': self.template_name,
            'extra_context': {
                'project': self.project,
            },
        }
        return diff_file(self.request, **diff_info)


class RepositoryFileRaw(RepositoryView):
    """
    This view returns FileNode from repository as file attachment.
    """

    def response(self, request, username, project_slug, revision, rel_repo_url):
        if self.has_errors:
            return self.get_error_response()
        node = self.project.repository.request(rel_repo_url, revision)
        response = HttpResponse(node.content, mimetype=node.mimetype)
        response['Content-Disposition'] = 'attachment; filename=%s' % node.name
        return response


class RepositoryFileAnnotate(RepositoryView):

    template_name='projector/project/repository/annotate.html'

    def response(self, request, username, project_slug, revision, rel_repo_url):
        if self.has_errors:
            return self.get_error_response()
        repo_info = {
                'repository': self.project.repository,
                'revision': revision,
                'node_path': rel_repo_url,
                'template_name': self.template_name,
                'extra_context': {
                    'project': self.project,
                },
            }
        return browse_repository(self.request, **repo_info)


class RepositoryChangesetList(RepositoryView):

    template_name = 'projector/project/repository/changeset_list.html'

    def response(self, request, username, project_slug):
        if self.has_errors:
            return self.get_error_response()
        context = {
            'project': self.project,
        }
        context['repository'] = self.project.repository
        context['CHANGESETS_PAGINATE_BY'] = \
            self.project.config.changesets_paginate_by
        return context


class RepositoryChangesetDetail(RepositoryView):

    template_name = 'projector/project/repository/changeset_detail.html'

    def response(self, request, username, project_slug, revision):
        if self.has_errors:
            return self.get_error_response()
        diff_info = {
            'repository': self.project.repository,
            'revision': revision,
            'template_name': self.template_name,
            'extra_context': {
                'project': self.project,
            }
        }
        return diff_changeset(self.request, **diff_info)





from django.db.models import Count
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext as _
from django.contrib import messages

from projector.models import Milestone
from projector.views.project import ProjectView
from projector.forms import MilestoneForm

class MilestoneListView(ProjectView):

    template_name = 'projector/project/milestones/home.html'

    def response(self, request, username, project_slug):
        milestone_list = self.project.milestone_set\
            .annotate(Count('task'))\
            .order_by('-created_at')
        context = {
            'project': self.project,
            'milestone_list': milestone_list,
        }
        return context

class MilestoneDetailView(ProjectView):
    """
    Returns milestone detail view.
    """

    template_name = 'projector/project/milestones/detail.html'

    def response(self, request, username, project_slug, milestone_slug):
        milestone = get_object_or_404(Milestone,
            project=self.project, slug=milestone_slug)
        context = {
            'project': self.project,
            'milestone': milestone,
        }
        return context

class MilestoneCreateView(ProjectView):
    """
    Adds milestone for project.
    """

    template_name = 'projector/project/milestones/add.html'
    perms = ['view_project', 'change_project']

    def response(self, request, username, project_slug):
        milestone = Milestone(project=self.project, author=request.user)
        form = MilestoneForm(request.POST or None, instance=milestone)
        if request.method == 'POST' and form.is_valid():
            milestone = form.save()
            msg = _("Milestone added successfully")
            messages.success(request, msg)
            return redirect(milestone.get_absolute_url())
        context = {
            'project': self.project,
            'form': form,
        }
        return context

class MilestoneEditView(ProjectView):
    """
    Edits chosen milestone.
    """

    template_name = 'projector/project/milestones/edit.html'
    perms = ['view_project', 'change_project']

    def response(self, request, username, project_slug, milestone_slug):
        milestone = get_object_or_404(Milestone, slug=milestone_slug)
        form = MilestoneForm(request.POST or None, instance=milestone)
        if request.method == 'POST' and form.is_valid():
            milestone = form.save()
            msg = _("Milestone updated successfully")
            messages.success(request, msg)
            return redirect(milestone.get_absolute_url())
        context = {
            'project': self.project,
            'form': form,
        }
        return context




"""
Unit tests for ``django-projector``

Won't run if we let setuptools to install dependancies inside current
directory - we need to install external packages before running test
suite.
"""
import os
import sys

os.environ["DJANGO_SETTINGS_MODULE"] = "example_project.settings_test"
from example_project import settings_test as settings

def main():
    sys.path.insert(0, os.path.abspath(os.path.curdir))
    from django.test.utils import get_runner
    test_runner = get_runner(settings)(verbosity=1)

    failures = test_runner.run_tests(['projector'])
    sys.exit(failures)

if __name__ == '__main__':
    main()




from django.test import TestCase
from django.template import Template, Context

from projector.settings import get_config_value

def render(template, context):
    """
    Returns rendered ``template`` with ``context``, which are given as string
    and dict respectively.
    """
    t = Template(template)
    return t.render(Context(context))


class HideEmailTest(TestCase):
    """
    Tests against hide_email filter.
    """

    def setUp(self):
        self.sub = get_config_value('HIDDEN_EMAIL_SUBSTITUTION')

    def test_simple(self):

        template = ''.join((
            '{% load projector_tags %}',
            '{{ author|hide_email }}',
        ))
        context = {'author': 'Joe Doe [joe.doe@example.com]'}
        output = render(template, context)

        self.assertEqual(output, 'Joe Doe [%s]' % (self.sub))

    def test_with_parameter(self):

        template = ''.join((
            '{% load projector_tags %}',
            '{{ author|hide_email:"HIDDEN_EMAIL" }}',
        ))
        context = {'author': 'Joe Doe [joe.doe@example.com]'}
        output = render(template, context)

        self.assertEqual(output, 'Joe Doe [%s]' % ('HIDDEN_EMAIL'))

    def test_multiple_simple(self):

        template = ''.join((
            '{% load projector_tags %}',
            '{% for author in authors %}'
            '{{ author|hide_email }}\n',
            '{% endfor %}',
        ))
        authors = ['Joe Doe [joe.doe@example.com]', 'Jack (jack@example.com)']
        context = {'authors': authors}
        output = render(template, context)

        self.assertEqual(output,
            'Joe Doe [%(sub)s]\n'
            'Jack (%(sub)s)\n' % {'sub': self.sub})

    def test_multiple_with_parameter(self):

        template = ''.join((
            '{% load projector_tags %}',
            '{% for author in authors %}'
            '{{ author|hide_email:"HIDDEN_EMAIL" }}\n',
            '{% endfor %}',
        ))
        authors = ['Joe Doe [joe.doe@example.com]', 'Jack (jack@example.com)']
        context = {'authors': authors}
        output = render(template, context)

        self.assertEqual(output,
            'Joe Doe [%(sub)s]\n'
            'Jack (%(sub)s)\n' % {'sub': 'HIDDEN_EMAIL'})




from django.views.generic import simple

def home(request, template_name='projector/home.html'):
    """
    Home view for projector app.
    """

    return simple.direct_to_template(request, {'template': template_name})




import datetime

from django.shortcuts import get_object_or_404, redirect
from django.core.exceptions import PermissionDenied
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.utils.translation import ugettext as _
from django.utils.decorators import method_decorator

from projector.models import Task
from projector.forms import TaskForm, TaskEditForm
from projector.filters import TaskFilter
from projector.views.project import ProjectView

from richtemplates.shortcuts import get_json_response

login_required_m = method_decorator(login_required)

class TaskListView(ProjectView):
    """
    Task for project listing view.
    """

    template_name='projector/project/task/list.html'
    perms_private = ['view_project', 'can_view_tasks']

    def response(self, request, username, project_slug):
        task_list = Task.objects.filter(project__id=self.project.id)\
                .select_related('priority', 'status', 'author', 'project')
        filters = TaskFilter(self.request.GET, queryset=task_list,
            project=self.project)
        if self.request.GET and 'id' in self.request.GET and \
                self.request.GET['id'] and filters.qs.count() == 1:
            task = filters.qs[0]
            messages.info(self.request, _("One task matched - redirecting..."))
            return redirect(task.get_absolute_url())
        self.context['filters'] = filters
        return self.context


class TaskListDataView(ProjectView):
    """
    Tasks data for charts.
    """

    def response(self, request, username, project_slug):
        import random
        import pyofc2
        t = pyofc2.title(text=_("Tasks"))
        b1 = pyofc2.bar()
        b1.values = range(10)
        b2 = pyofc2.bar()
        b2.values = [random.randint(0,9) for i in range(9)]
        b2.colour = '#56acde'
        chart = pyofc2.open_flash_chart()
        chart.title = t
        chart.add_element(b1)
        chart.add_element(b2)
        return HttpResponse(chart.render())


class TaskDetailView(ProjectView):
    """
    Task detail view.
    Users may update task here.
    """

    template_name = 'projector/project/task/detail.html'
    perms_private = ['view_project', 'can_view_tasks']
    perms_POST = ['can_change_task']

    def response(self, request, username, project_slug, task_id):
        task = get_object_or_404(
            Task.objects.select_related('type', 'priority', 'status', 'owner',
                'author', 'editor', 'milestone', 'component', 'project'),
            id = task_id,
            project__author__username = username,
            project__slug = project_slug)

        self.context['task'] = task
        self.context['is_watched'] = task.is_watched(request.user)
        self.context['now'] = datetime.datetime.now()

        return self.context

class TaskCreateView(ProjectView):
    """
    New Task creation view.
    """

    template_name = 'projector/project/task/create.html'
    perms = ProjectView.perms + ['can_add_task']

    @login_required_m
    def response(self, request, username, project_slug):
        initial = {
            'owner': request.user.username, # form's owner is UserByNameField
        }
        instance = Task.objects.get_for_project(self.project)
        instance.author = request.user
        instance.author_ip = request.META.get('REMOTE_ADDR', '')
        instance.editor = instance.author
        instance.editor_ip = instance.author_ip

        form = TaskForm(request.POST or None, initial=initial, instance=instance)

        if request.method == 'POST':
            if form.is_valid():
                task = form.save(
                    editor = request.user,
                    editor_ip = request.META.get('REMOTE_ADDR', ''),
                )
                task.create_revision()
                messages.success(request, _("Task created succesfully."))
                task.notify()
                return redirect(task.get_absolute_url())

        self.context['form'] = form

        return self.context

class TaskEditView(ProjectView):
    """
    Edit Task meta information. task_detail edits the rest.
    """

    template_name = 'projector/project/task/create.html'
    perms = ['can_change_task']

    @login_required_m
    def response(self, request, username, project_slug, task_id):
        task = get_object_or_404(Task, id=task_id,
            project__author__username=username, project__slug=project_slug)

        if request.method == 'POST':
            form = TaskEditForm(request.POST, instance=task)
            if form.is_valid():
                if request.user.is_authenticated():
                    editor = request.user
                else:
                    editor = User.get_anonymous() # available from guardian
                task = form.save(
                    editor=editor,
                    editor_ip=request.META.get('REMOTE_ADDR', ''),
                )
                task.create_revision()
                messages.success(request, _("Task updated successfully."))
                task.notify()
                return redirect(task.get_absolute_url())
        else:
            form = TaskEditForm(instance=task, initial={
                'owner': task.owner and task.owner.username or None,
            })

        self.context['form'] = form

        return self.context

class TaskWatchView(ProjectView):
    """
    Makes request's user watching this task.
    """

    perms_private = ['view_project', 'can_view_tasks']

    def response(self, request, username, project_slug, task_id):
        task = get_object_or_404(Task, id=task_id,
            project__author__username=username, project__slug=project_slug)
        if request.method == 'POST':
            task.watch(request.user)
            if request.is_ajax():
                return get_json_response()
            return redirect(task.get_absolute_url())
        else:
            # Only POST methods are allowed here
            raise PermissionDenied()

class TaskUnwatchView(ProjectView):
    """
    Makes request's user watching this task.
    """

    perms_private = ['view_project', 'can_view_tasks']

    def response(self, request, username, project_slug, task_id):
        task = get_object_or_404(Task, id=task_id,
            project__author__username=username, project__slug=project_slug)
        if request.method == 'POST':
            if request.is_ajax():
                return get_json_response()
            task.unwatch(request.user)
            return redirect(task.get_absolute_url())
        else:
            # Only POST methods are allowed here
            raise PermissionDenied()




from django.test.client import Client
from django.contrib.auth.models import User

from projector.tests.base import ProjectorTestCase
from projector.models import Project, Component

class ComponentTest(ProjectorTestCase):

    def setUp(self):
        self.client = Client()
        cred = 'componenter'
        self.user, created = User.objects.get_or_create(
            username = cred,
            email = 'componenter@example.com',
            is_superuser = True,
            is_active = True)
        self.user.set_password(cred)
        self.user.save()
        self.user._plain_password = cred
        self.project = Project.objects.create_project(
            name = 'component-test-project',
            slug = 'component-test-project',
            author = self.user,
        )

    def test_name_uniqueness(self):
        self.client.login(username = self.user.username,
            password = self.user._plain_password)

        new_component_name = u'component-1'
        self._get_response(
            url = self.project.get_component_add_url(),
            data = {'name': new_component_name},
            method = 'POST', follow = True)

        # By default one component is already defined so now there should be 2
        self.assertEqual(self.project.component_set.count(), 2)

        response = self._get_response(
            url = self.project.get_component_add_url(),
            data = {'name': new_component_name},
            method = 'POST', follow = True)

        # Should not be added as it has same name
        self.assertEqual(self.project.component_set.count(), 2)
        name_field_errors = response.context['form']._errors['name']
        self.assertTrue(len(name_field_errors) > 0,
            "Component with name '%s' for this project already exist "
            "so we shouldn't be able to add new one" % new_component_name)

        another_name = u'component-2'
        self._get_response(
            url = self.project.get_component_add_url(),
            data = {'name': another_name},
            method = 'POST', follow = True)
        self.assertEqual(self.project.component_set.count(), 3)

        # Should not allow to update name of the component to one already
        # existing
        component = Component.objects.get(project = self.project,
            name = another_name)
        response = self._get_response(
            url = component.get_edit_url(),
            data = {'name': new_component_name},
            method = 'POST', follow = True)
        name_field_errors = response.context['form']._errors['name']
        self.assertTrue(len(name_field_errors) > 0,
            "Component with name '%s' for this project already exist "
            "so we shouldn't be able to change old name '%s' to this one"
            % (new_component_name, component.name))







import logging

from django.contrib import admin
from django.utils.translation import ugettext as _

from projector.models import Status, Task, Priority,\
    Project, ProjectCategory, TaskType, Milestone, Component,\
    TaskRevision
from projector.models import Membership, Team

from richtemplates.forms import LimitingModelForm

from django_extensions.admin import ForeignKeyAutocompleteAdmin

def author_editor_save_model(self, request, obj, form, change):
    """
    Overrides standard admin.ModelAdmin save_model method.
    It sets editor (and his/her IP) based on data from requet.
    If model is new (it's primary key is not yet set) sets
    author/author_ip fields too.
    """

    logging.debug("%s.save_model method called!" % self.__class__.__name__)

    if getattr(obj, 'pk', None) is None:
        # Its a new instance so we need to
        # set author and author_ip fields
        obj.author = request.user
        obj.author_ip = request.META.get('REMOTE_ADDR', '')
    obj.editor = request.user
    obj.editor_ip = request.META.get('REMOTE_ADDR', '')

    return super(self.__class__, self).save_model(request, obj, form, change)

class StatusAdmin(admin.ModelAdmin):
    list_display = ( 'id', 'name', 'order', 'is_resolved', 'project')
    list_display_links = ('id',)
    list_editable = ( 'name', 'order', 'is_resolved', )
    list_filter = ('project',)
    readonly_fields = ['project']

class TaskAdminForm(LimitingModelForm):
    class Meta:
        choices_limiting_fields = ['project']
        model = Task

class TaskRevisionInline(admin.TabularInline):
    model = TaskRevision
    extra = 1

class TaskAdmin(admin.ModelAdmin):
    list_display = ( 'project', 'component', 'id', 'summary',
        'created_at', 'author', 'status', 'priority', 'type')
    list_display_links = ('summary',)
    list_filter = ('project',)
    date_hierarchy = 'created_at'
    save_on_top = True
    search_fields = ['id', 'summary', 'description']
    form = TaskAdminForm

    fieldsets = (
        (_("Task detail"), {
            'fields': (
                'summary',
                ('project', 'component'),
                'description',
                'status',
                'priority',
                'type',
                'owner',
                'milestone',
                'deadline',
            )
        }),
        (_("Author & editor"), {
            'classes': ['collapsed collapse-toggle'],
            'fields': (
                ('author', 'author_ip'),
                ('editor', 'editor_ip'),
            ),
        }),
    )

    # This option would be used from Django 1.2
    readonly_fields = ('author_ip', 'editor_ip')

    class Media:
        css = {
            'all': ['projector/css/monoarea.css'],
        }

class TaskAutocompleteAdmin(TaskAdmin, ForeignKeyAutocompleteAdmin):
    related_search_fields = {
        'owner': ('username',),
    }

class OrderedDictModelAdmin(admin.ModelAdmin):
    list_display = ( 'id', 'name', 'order', 'description' )
    list_display_links = ( 'id', 'name' )
    list_editable = ( 'order', )

class MilestoneInline(admin.TabularInline):
    model = Milestone
    extra = 1

class TaskTypeInline(admin.TabularInline):
    model = TaskType
    extra = 1

class ComponentInline(admin.TabularInline):
    model = Component
    extra = 1

class StatusInline(admin.TabularInline):
    model = Status
    extra = 1

class PriorityInline(admin.TabularInline):
    model = Priority
    extra = 1

class ProjectAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'category', 'home_page_url', 'is_active',
        'public', 'author', 'created_at', 'outdated')
    list_display_links = ('name',)
    list_filter = ('category', 'is_active', 'public', 'outdated')
    save_on_top = True
    search_fields = ['name', 'description']

    inlines = [MilestoneInline, ComponentInline, TaskTypeInline,
        StatusInline, PriorityInline]

    actions = ['make_public', 'make_private']

    def make_public(self, request, queryset):
        queryset.update(public=True)
    make_public.short_description = _("Make all selected projects public")

    def make_private(self, request, queryset):
        queryset.update(public=False)
    make_private.short_description = _("Make all selected projects private")

class ComponentAdmin(admin.ModelAdmin):
    list_display = 'project', 'name', 'description'
    list_display_links = 'name',
    list_filter = 'project',
    search_fields = ['project', 'name']


class TaskTypeAdmin(admin.ModelAdmin):
    list_display = ['id', 'project', 'name', 'order']
    list_display_links = ['name']
    list_filter = ['project']
    search_field = ['name', 'project']


admin.site.register(Status, StatusAdmin)
#admin.site.register(Priority, OrderedDictModelAdmin)
admin.site.register(TaskType, TaskTypeAdmin)
admin.site.register(Task, TaskAutocompleteAdmin)
admin.site.register(Project, ProjectAdmin)
admin.site.register(ProjectCategory)
admin.site.register(Membership)
admin.site.register(Team)
#admin.site.register(Component)
#admin.site.register(Milestone) # Should be maintain with project itself




from django.test.client import Client
from django.contrib.auth.models import User

from projector.tests.base import ProjectorTestCase
from projector.models import Project
from projector.utils import str2obj
from projector.settings import get_config_value

statuses = str2obj(get_config_value('DEFAULT_PROJECT_WORKFLOW')).statuses

class StatusTest(ProjectorTestCase):

    def setUp(self):
        self.client = Client()
        cred = 'statuser'
        self.user = User.objects.create(
            username = cred,
            email = 'statuser@example.com',
            is_superuser = True,
            is_active = True)
        self.user.set_password(cred)
        self.user.save()
        self.user._plain_password = cred
        self.project = Project.objects.create_project(
            name = 'status-test-project',
            slug = 'status-test-project',
            author = self.user,
        )

    def test_name_uniqueness(self):
        self.client.login(username = self.user.username,
            password = self.user._plain_password)

        order = len(statuses) + 1
        new_status_name = u'status-1'
        self._get_response(
            url = self.project.get_workflow_add_status_url(),
            data = {'name': new_status_name, 'order': unicode(order)},
            method = 'POST', follow = True)
        order += 1

        self.assertEqual(self.project.status_set.count(), len(statuses) + 1)

        self._get_response(
            url = self.project.get_workflow_add_status_url(),
            data = {'name': new_status_name, 'order': unicode(order)},
            method = 'POST', follow = True)

        # TODO: Need to test editing statuses' names




import sys

from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

projector = __import__('projector')
readme_file = 'README.rst'
try:
    long_description = open(readme_file).read()
except IOError, err:
    sys.stderr.write("[ERROR] Cannot find file specified as "
        "``long_description`` (%s)\n" % readme_file)
    sys.exit(1)

setup(
    name = 'django-projector',
    version = projector.__version__,
    url = 'http://www.django-projector.org',
    author = 'Lukasz Balcerzak',
    author_email = 'lukasz.balcerzak@python-center.pl',
    description = projector.__doc__,
    long_description = long_description,
    zip_safe = False,
    packages = find_packages(),
    include_package_data = True,
    scripts = [],
    requires = ['Djalog', 'richtemplates', 'mercurial'],
    install_requires = [
        'djalog',
        'django-richtemplates',
        'django-pagination>=1.0.7',
        'django-autoslug',
        'django-celery',
        'django-guardian',
        'django-registration',
        'docutils',
        'south',
    ],
    classifiers = ['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
    ],
    test_suite='tests.main',
)




from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import string_concat, ugettext as _
from django.contrib import messages
import logging

try:
    import reportlab
    from reportlab.graphics.shapes import Drawing              
    from reportlab.lib import colors
    from reportlab.graphics.charts.piecharts import Pie, Pie3d
except ImportError:
    reportlab = None

from projector.models import Task, Status

def task_reports(request, template_name='projector/task/reports.html'):
    """
    Task reports view.
    """
    #task_list = Task.objects.all()
    return get_actual_status_img(request)

def get_actual_status_img(request, output_format='png', width=600, height=600):
    if reportlab is None:
        messages.error(request, _("Module") + " reportlab " + _("is not available"))
        try:
            redirect_to = request.META['HTTP_REFERER']
        except KeyError:
            redirect_to = '/'
        return HttpResponseRedirect(redirect_to)
    status_list = Status.objects.all()
    
    drawing = Drawing(width, height)
    pie = Pie3d()
    pie.x = 100
    pie.y = 100
    pie.width = width / 2
    pie.height = height / 2

    pie.labels = [ s.name for s in status_list ]
    pie.data = [ s.task_set.count() for s in status_list ]
    pie.slices[3].fontColor = colors.red
    pie.slices[0].fillColor = colors.darkcyan
    pie.slices[1].fillColor = colors.blueviolet
    pie.slices[2].fillColor = colors.blue
    pie.slices[3].fillColor = colors.cyan
    pie.slices[4].fillColor = colors.aquamarine
    pie.slices[5].fillColor = colors.cadetblue
    pie.slices[6].fillColor = colors.lightcoral
        
    drawing.add(pie)
    image = drawing.asString(output_format)
    response = HttpResponse(image, mimetype='image/%s' % output_format.lower())
    return response



import logging

from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext as _
from django.contrib import messages
from django.contrib.admin.util import NestedObjects
from django.contrib.contenttypes.models import ContentType

from projector.models import Membership
from projector.views.project import ProjectView
from projector.forms import MembershipForm, ProjectMembershipPermissionsForm
from projector.forms import MembershipDeleteForm

from guardian.models import UserObjectPermission
from guardian.shortcuts import get_perms

class MemberListView(ProjectView):
    """
    Returns members view.
    """

    template_name = 'projector/project/members/home.html'

    def response(self, request, username, project_slug):
        memberships = Membership.objects.filter(project=self.project)
        context = {
            'project': self.project,
            'memberships': memberships,
        }
        return context

class MemberAddView(ProjectView):
    """
    Adds member for a project.
    """

    template_name = 'projector/project/members/add.html'
    perms = ['view_project', 'add_member_project']

    def response(self, request, username, project_slug):
        membership = Membership(
            project = self.project,
        )
        form = MembershipForm(request.POST or None, instance=membership)

        if request.method == 'POST' and form.is_valid():
            logging.info("Saving member %s for project '%s'"
                % (form.instance.member, form.instance.project))
            form.save()
            return redirect(self.project.get_members_url())
        elif form.errors:
            logging.error("Form contains errors:\n%s" % form.errors)

        context = {
            'project': form.instance.project,
            'form': form,
        }
        return context

class MemberEditView(ProjectView):
    """
    Manages membership settings and permissions of project's member.
    """

    template_name = 'projector/project/members/edit.html'
    perms = ['view_project', 'can_change_member']

    def response(self, request, username, project_slug, member_username):
        membership = get_object_or_404(
            Membership.objects.select_related('project', 'member'),
            project__slug=project_slug,
            project__author__username=username,
            member__username=member_username)\

        member = membership.member
        project = membership.project
        if project.author == member:
            messages.warning(request, _("Project owner's membership cannot be "
                "modified. He/She has all permissions for this project."))
            return redirect(project.get_members_url())
        member_perms = get_perms(member, project)

        form = ProjectMembershipPermissionsForm(request.POST or None,
            membership = membership,
            initial_permissions = member_perms,
            request = request)
        if request.method == 'POST':
            if form.is_valid():
                logging.info("Form's data:\n%s" % form.cleaned_data)
                messages.success(request, _("Permissions updated"))
                form.save()
            else:
                messages.error(request,
                    _("Errors occured while processing the form"))
            return redirect(membership.get_absolute_url())
        context = {
            'project': self.project,
            'form': form,
            'membership': membership,
            'member_perms': member_perms,
        }
        return context

class MemberDeleteView(ProjectView):
    """
    Removes member from project.
    """

    template_name = 'projector/project/members/delete.html'
    perms = ['view_project', 'delete_member_project']

    def response(self, request, username, project_slug, member_username):
        membership = get_object_or_404(
            Membership.objects.select_related('project', 'member'),
            project__slug=project_slug,
            project__author__username=username,
            member__username=member_username)\

        member = membership.member
        project = membership.project

        if project.author == member:
            messages.warning(request, _("Project owner's membership cannot be "
                "removed."))
            return redirect(project.get_members_url())
        collector = NestedObjects()
        membership._collect_sub_objects(collector)
        form = MembershipDeleteForm(request.POST or None)
        perms_to_delete = UserObjectPermission.objects.filter(
            user = member,
            content_type = ContentType.objects.get_for_model(project),
            object_id = project.id)

        if request.method == 'POST':
            # Confirm removal
            if form.is_valid() and project.author != member:
                msg = _("Membership removed")
                messages.success(request, msg)
                membership.delete()
                perms_to_delete.delete()
                return redirect(project.get_members_url())
            else:
                msg = _("Couldn't remove membership")
                messages.error(request, msg)
                if project.author == member:
                    msg = _("Project's author cannot be removed")
                    messages.error(request, msg)
        context = {
            'project': project,
            'membership': membership,
            'form': form,
            'to_delete': collector.nested(),
            'member_perms': perms_to_delete,
        }
        return context




from django import forms
from django.utils.translation import ugettext as _

from projector.core.exceptions import ForkError
from projector.forks.base import BaseExternalForkForm
from projector.models import Project

from vcs.exceptions import VCSError

class BitbucketForkForm(BaseExternalForkForm):

    site = 'bitbucket.org'
    help_text = _('Only public projects are allowed to be forked from '
                  'bitbucket')

    username = forms.RegexField(regex=r'^[-\w]+$', max_length=128,
        label=_('Username'),
        help_text=_('Name of user you want to fork project from'))
    projectname = forms.RegexField(regex=r'^[-\w]+$',max_length=128,
        label=_('Project name'),
        help_text=_('Name of project you want to fork belonging to specified '
                    'user'))
    use_https = forms.BooleanField(initial=False, label=_('Use https'),
        help_text=_('If checked, would clone project using secured '
                    'connection'), required=False)

    def clean_projectname(self):
        data = self.cleaned_data['projectname']
        if getattr(self, 'request', None) is not None:
            try:
                Project.objects.get(name=data, author=self.request.user)
                raise forms.ValidationError(_("Project with same name already "
                                              "exists"))
            except Project.DoesNotExist:
                pass
        return data

    def fork(self):
        """
        This method only creates ``Project`` instance with proper attributes
        as real fork is done by ``Project``'s ``post_save`` handler.
        """
        data = self.cleaned_data
        url = self.get_url()
        try:
            project = Project.objects.create_project(
                vcs_alias = 'hg',
                author = self.request.user,
                name = data['projectname'],
                public = self.is_public(),
                fork_url = url)
        except VCSError:
            raise ForkError(_("Error during fork procedure"))
        return project

    def get_url(self):
        if self.cleaned_data.get('use_https', False):
            schema = 'https'
        else:
            schema = 'http'
        url = '://'.join((schema, self.site))
        username = self.cleaned_data['username']
        projectname = self.cleaned_data['projectname']
        url = '/'.join((url, username, projectname))
        return url




"""
Exceptions raised by ``django-projector``.

All internal error classes should extend from one of those exceptions.

:error:`ProjectorError` should always be *top-level* exception class.
"""

class ProjectorError(Exception):
    """
    Main ``django-projector`` exception.
    """
    pass

class ConfigAlreadyExist(ProjectorError):
    pass

class ForkError(ProjectorError):
    pass




import django_filters

from django import forms
from django.utils.translation import ugettext as _

from projector.models import Task

class ResolvedStatusFilter(django_filters.BooleanFilter):
    field_class = forms.NullBooleanField

    def __init__(self, *args, **kwargs):
        super(ResolvedStatusFilter, self).__init__(*args, **kwargs)
        self.field.widget.choices[0] = (u'1', _('All'))

    def filter(self, qs, value):
        if value is not None:
            return qs.filter(status__is_resolved=value)
        return qs

def TaskFilter(data=None, queryset=Task.objects.all(), project=None):
    """
    Factory method which returns ``FilterSet`` for given project.
    """
    class TaskFilter(django_filters.FilterSet):
        is_resolved = ResolvedStatusFilter()
        class Meta:
            model = Task
            fields = ['id', 'priority', 'milestone', 'status', 'component',
                'owner', 'type']

        def __init__(self, *args, **kwargs):
            super(TaskFilter, self).__init__(*args, **kwargs)
            if project:
                self.filters['status'].extra.update(
                    {'queryset': project.status_set.all()})
                self.filters['priority'].extra.update(
                    {'queryset': project.priority_set.all()})
                self.filters['milestone'].extra.update(
                    {'queryset': project.milestone_set.all()})
                self.filters['component'].extra.update(
                    {'queryset': project.component_set.all()})
                self.filters['owner'].extra.update(
                    {'queryset': project.members.order_by('username')})
                self.filters['type'].extra.update(
                    {'queryset': project.tasktype_set.all()})
                #self.filters['owner'].extra.update(
                #    {'label': _('Assigned to')})
    filterset = TaskFilter(data, queryset)
    return filterset




from settings import *

# Make celery fire up tasks synchronously
CELERY_ALWAYS_EAGER = True

PROJECTOR_DEFAULT_PROJECT_WORKFLOW = 'projector.tests.workflow.TestWorkflow'
PROJECTOR_PROJECTS_HOMEDIR_GETTER = 'projector.tests.base.get_homedir'

# Hide logging messages
DJALOG_LEVEL = 50




from django import forms
from django.utils.translation import ugettext_lazy as _

def IntegerValidator(min, max):
    """
    Factory function, returns validator method processing integers.
    """
    def _validator(value):
        try:
            int(value)
            if value < min:
                raise forms.ValidationError(_("Minimum is %d" % min))
            elif value > max:
                raise forms.ValidationError(_("Maximum is %d" % max))
        except TypeError:
            raise forms.ValidationError(_("Must be a number"))
        return value
    return _validator





class LazyProperty(object):
    """
    Decorator for easier creation of ``property`` from potentially expensive to
    calculate attribute of the class.

    Usage::

      class Foo(object):
          @LazyProperty
          def bar(self):
              print 'Calculating self._bar'
              return 42

    Taken from http://blog.pythonisito.com/2008/08/lazy-descriptors.html.
    """


    def __init__(self, func):
        self._func = func
        self.__name__ = func.__name__
        self.__doc__ = func.__doc__

    def __get__(self, obj, klass=None):
        if obj is None: return None
        result = obj.__dict__[self.__name__] = self._func(obj)
        return result




import logging

from django import forms
from django.forms.models import modelformset_factory
from django.forms.models import BaseModelFormSet
from django.utils.translation import ugettext as _
from django.contrib import messages
from django.contrib.auth.models import User, Group
from django.contrib.formtools.wizard import FormWizard
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect

from guardian.shortcuts import assign, remove_perm, get_perms,\
    get_perms_for_model

from projector.core.exceptions import ProjectorError
from projector.forks.base import BaseExternalForkForm
from projector.models import Membership
from projector.models import Team
from projector.models import Project
from projector.models import Config
from projector.models import Task
from projector.models import Status
from projector.models import Component
from projector.models import Milestone
from projector.models import UserProfile
from projector.settings import get_config_value
from projector.signals import setup_project
from projector.utils.basic import str2obj

from richtemplates.forms import LimitingModelForm, RestructuredTextAreaField,\
    ModelByNameField, UserByNameField
from richtemplates.widgets import RichCheckboxSelectMultiple
from richtemplates.forms import RichSkinChoiceField, RichCodeStyleChoiceField

from vcs.backends import get_supported_backends


PUBLIC_RADIO_CHOICES = [
    (u'public', _("Public")),
    (u'private', _("Private"))
]

class PerProjectUniqueNameMixin(object):
    """
    Can be used for forms with model which is related with Project and has
    ``name`` field which should be unique per project.
    """
    def clean_name(self):
        name = self.cleaned_data['name']
        if self._meta.model.objects\
            .filter(project=self.instance.project)\
            .filter(name__iexact=name)\
            .exists():
            raise forms.ValidationError(_("%(class)s with same name already "
                "defined for this project" % {
                    'class': self._meta.model.__name__}))
        return name

enabled_backends = get_config_value('ENABLED_VCS_BACKENDS')
supported_backends = get_supported_backends()
if not set(enabled_backends).issubset(set(supported_backends)):
    raise ImproperlyConfigured("VCS supports only following backends: %s"
        % ', '.join(supported_backends))

VCS_BACKENDS_CHOICES = ((key, key) for key in enabled_backends)

class ProjectBaseForm(forms.ModelForm):
    name = forms.CharField(min_length=2, max_length=64, label=_('Name'))
    public = forms.ChoiceField(label=_("Visibility"),
        choices=PUBLIC_RADIO_CHOICES,
        widget=forms.RadioSelect(),
        initial=u'private',
    )
    class Meta:
        model = Project
        exclude = ('members', 'author', 'editor', 'repository', 'teams',
            'parent', 'fork_url', 'is_active', 'category', 'state',
            'error_text')

    def __init__(self, *args, **kwargs):
        form = super(ProjectBaseForm, self).__init__(*args, **kwargs)
        # Update ``status`` field while creating new task
        if get_config_value('PRIVATE_ONLY'):
            self.fields['public'].choices.pop(0)
        return form

    def clean_public(self):
        data = self.cleaned_data['public']
        # Returned data depends on PUBLIC_RADIO_CHOICES
        if data == u'public':
            return True
        elif data == u'private':
            return False
        else:
            raise forms.ValidationError(_("Choose one of the given options"))

    def clean_teams(self):
        # Not used with ``teams`` field excluded
        data = self.cleaned_data['teams']
        logging.info(data)
        teams = [Team(group=group) for group in
            data]
        return teams

    def clean_name(self):
        name = self.cleaned_data['name']
        if name.strip().lower() in get_config_value('BANNED_PROJECT_NAMES'):
            raise forms.ValidationError(_("This name is restricted"))
        if Project.objects\
                .filter(name=name, author=self.instance.author)\
                .exclude(pk=self.instance.pk).exists():
            msg = _("You have project with same name already")
            raise forms.ValidationError(msg)
        return name


class ProjectCreateForm(ProjectBaseForm):
    vcs_alias = forms.ChoiceField(choices=VCS_BACKENDS_CHOICES,
        label=_('Version Control Backend'),
        initial=get_config_value('DEFAULT_VCS_BACKEND'),
        widget=forms.HiddenInput) # Hidden until more backends are supported

    def save(self, commit=True):
        instance = super(ProjectCreateForm, self).save(commit=False)
        if commit:
            instance.save()
            setup_project.send(sender=Project, instance=instance,
                vcs_alias=self.cleaned_data.get('vcs_alias', None))
            for team in self.cleaned_data.get('teams', ()):
                team.project = instance
                team.save()
        return instance


class ProjectEditForm(ProjectBaseForm):
    pass


class ProjectForkForm(forms.Form):
    """
    Currently dummy form which is enough for fork page.
    """
    pass


class ConfigForm(forms.ModelForm):
    changesets_paginate_by = forms.IntegerField(
        label=_('Changesets paginate by'), min_value=5, max_value=50)


    class Meta:
        model = Config
        exclude = ['project', 'editor', 'from_email_address']


class TaskForm(LimitingModelForm):
    owner = ModelByNameField(max_length=128, queryset=User.objects.all,
        attr='username', label=_('Owner'), required=False)
    deadline = forms.DateField(required=False, label=_("Deadline"),
        widget=forms.DateInput(attrs={'class': 'datepicker'}))
    watch_changes = forms.BooleanField(False, label=_('Watch for changes'),
        initial=True)

    class Meta:
        model = Task
        exclude = ['author', 'author_ip', 'project', 'editor', 'editor_ip']
        choices_limiting_fields = ['project']

    def __init__(self, *args, **kwargs):
        super(TaskForm, self).__init__(*args, **kwargs)
        if 'status' in self.fields:
            # Update ``status`` field while creating new task
            self.fields['status'].queryset = Status.objects.filter(
                project=self.instance.project, is_initial=True)
            self.fields['status'].empty_label=None
        #self.fields['owner'].queryset = self.instance.project.members.all()

    def save(self, editor, editor_ip, project=None, commit=True):
        assert project or self.instance.project,\
            "For new tasks you have to pass project object into this method."
        if project:
            self.instance.project = project
        self.instance.editor = editor
        self.instance.editor_ip = editor_ip
        if not self.instance.id:
            # For new tasks we basically treat editor as the author
            self.instance.author = editor
            self.instance.author_ip = editor_ip
        id = self.instance._calculate_id()
        logging.debug("Calculated id: %s" % id)
        task = super(TaskForm, self).save(commit)
        if commit and self.cleaned_data.get('watch_changes', False):
            task.watch(editor)
        return task


class TaskEditForm(TaskForm):
    deadline = forms.DateField(required=False, label=_("Deadline"),
        widget=forms.DateInput(attrs={'class': 'datepicker'}))
    description = RestructuredTextAreaField(max_length=3000,
        label=_('Description'))
    comment = RestructuredTextAreaField(max_length=3000,
        label=_('Comment'), widget=forms.Textarea, required=False)

    def __init__(self, *args, **kwargs):
        super(TaskEditForm, self).__init__(*args, **kwargs)
        if 'status' in self.fields:
            status_field = self['status'].field
            status_field.queryset = self.instance.status.destinations.all()
        if 'watch_changes' in self.fields:
            self.fields.pop('watch_changes')

    def clean(self):
        comment = self.cleaned_data.get('comment', None)
        logging.debug("Setting comment to: %s" % comment)
        self.instance.comment = comment
        return super(TaskEditForm, self).clean()


class MembershipForm(LimitingModelForm):
    member = ModelByNameField(max_length=128, queryset=User.objects.all,
        attr='username', label=_("Member"))

    class Meta:
        model = Membership
        exclude = ['project']
        choices_limiting_fields = ['project']

    def clean_member(self, commit=True):
        member = self.cleaned_data['member']
        if Membership.objects.filter(
            member = member,
            project = self.instance.project,
        ).count() > 0:
            raise forms.ValidationError(_("This user is already member of "
                "this project"))
        return member

def get_editable_perms():
    editable_perms = get_config_value('EDITABLE_PERMISSIONS')
    perms = [(p.codename, p.name) for p in get_perms_for_model(Project)
        if p.codename in editable_perms]
    perms.sort(key=lambda pair: pair[0])
    return perms


class MembershipDeleteForm(forms.Form):
    post = forms.BooleanField(initial=True, widget=forms.HiddenInput)


class ProjectMembershipPermissionsForm(forms.Form):
    permissions = forms.MultipleChoiceField(
        choices = (), # fetch for every instance (see __init__)
        label = _("Permissions"),
        widget = RichCheckboxSelectMultiple,
        required = False)

    def __init__(self, data=None, initial_permissions=[], membership=None,
            request=None, send_messages=False):
        super(ProjectMembershipPermissionsForm, self).__init__(data)
        self.membership = membership
        self['permissions'].field.choices = get_editable_perms()
        self.fields['permissions'].initial = initial_permissions
        self.request = request
        self.send_messages = send_messages

    def _message(self, level, message):
        assert level in ('success', 'warning', 'info', 'error')
        if self.request and self.send_messages:
            getattr(messages, level)(self.request, message)

    def save(self, commit=True):
        """
        Saves granted permissions and removes those switched off.
        """
        member, project = self.membership.member, self.membership.project

        granted_perms = self.cleaned_data['permissions']
        logging.info("Granted perms: %s" % granted_perms)
        member_perms = get_perms(member, project)
        # Grant permissions
        for perm in granted_perms:
            if perm not in member_perms:
                assign(perm, member, project)
                self._message('info', _("Permission added: %s" % perm))
        # Remove permissions
        for perm in member_perms:
            if perm not in granted_perms:
                remove_perm(perm, member, project)
                # notify user if perm is still granted by member's groups
                groups_with_perm = member.groups.filter(
                    groupobjectpermission__permission__codename=perm)
                if groups_with_perm:
                    messages.warning(self.request, _("Permission %(perm)s is "
                        "still granted for %(user)s as following group(s) "
                        "has/have it: %(groups)s" % {
                            'perm': perm,
                            'user': member,
                            'groups': ', '.join(
                                (str(group) for group in groups_with_perm)
                            )
                        }))
                self._message('warning', _("Permission removed: %s" % perm))


class ProjectTeamPermissionsForm(forms.Form):
    permissions = forms.MultipleChoiceField(
        choices = get_editable_perms(),
        label = _("Permissions"),
        widget = RichCheckboxSelectMultiple,
        required = False)

    def __init__(self, data=None, initial_permissions=[], team=None,
            request=None, send_messages=False):
        super(ProjectTeamPermissionsForm, self).__init__(data)
        self.team = team
        self.fields['permissions'].initial = initial_permissions
        self.request = request
        self.send_messages = send_messages

    def _message(self, level, message):
        assert level in ('success', 'warning', 'info', 'error')
        if self.request and self.send_messages:
            getattr(messages, level)(self.request, message)

    def save(self, commit=True):
        """
        Saves granted permissions and removes those switched off.
        """
        group, project = self.team.group, self.team.project

        granted_perms = self.cleaned_data['permissions']
        team_perms = get_perms(group, project)
        # Grant permissions
        for perm in granted_perms:
            if perm not in team_perms:
                assign(perm, group, project)
                self._message('info', _("Permission added: %s" % perm))
        # Remove permissions
        for perm in team_perms:
            if perm not in granted_perms:
                remove_perm(perm, group, project)
                self._message('warning', _("Permission removed: %s" % perm))


class TeamForm(LimitingModelForm):
    group = ModelByNameField(queryset=Group.objects.all,
        max_length=64, label=_("Group"))

    class Meta:
        model = Team
        exclude = ['project']
        choices_limiting_fields = ['project']

    def clean_group(self):
        group = self.cleaned_data['group']
        if Team.objects.filter(
            group = group,
            project = self.instance.project,
        ).count() > 0:
            raise forms.ValidationError(_("This group is already a team "
                "of this project"))
        return group


class TeamDeleteForm(forms.Form):
    post = forms.BooleanField(initial=True, widget=forms.HiddenInput)


class MilestoneForm(forms.ModelForm, PerProjectUniqueNameMixin):
    deadline = forms.DateField(required=False, label=_("Deadline"),
        widget=forms.DateInput(attrs={'class': 'datepicker'}))

    class Meta:
        model = Milestone
        exclude = ['project', 'author']


class ComponentForm(forms.ModelForm, PerProjectUniqueNameMixin):

    class Meta:
        model = Component
        exclude = ['project']


class StatusEditForm(forms.ModelForm, PerProjectUniqueNameMixin):

    class Meta:
        model = Status
        exclude = ['project']


class StatusForm(StatusEditForm):

    class Meta:
        model = Status
        exclude = ['project', 'destinations']


class BaseStatusFormSet(BaseModelFormSet):

    def clean(self):
        """
        Checks that no two statuses have the same name.
        """
        if any(self.errors):
            # Don't bother validating the formset unless each form is valid on
            # its own
            return
        # TODO: Notify user which forms were posted with same names (use
        # form._errors['name'] = ... )
        names = []
        for form in self.forms:
            name = form.cleaned_data['name']
            if name in names:
                raise forms.ValidationError(_("Statuses in a set must have "
                    "distinct name"))
            names.append(name)

    def add_fields(self, form, index):
        super(BaseStatusFormSet, self).add_fields(form, index)
        qs = form['destinations'].field.queryset
        if form.instance.project:
            qs = qs.filter(project = form.instance.project)
            form['destinations'].field.queryset = qs

StatusFormSet = modelformset_factory(Status,
    exclude = ['description', 'project'],
    extra = 0,
    formset = BaseStatusFormSet,
)

class UserProfileForm(forms.ModelForm):
    skin = RichSkinChoiceField()
    code_style = RichCodeStyleChoiceField()

    class Meta:
        model = UserProfile
        exclude = ('user', 'activation_token', 'is_team', 'group')


class UserConvertToTeamForm(forms.Form):
    confirm = forms.BooleanField(label=_('confirm'))

    def _get_user(self):
        return getattr(self, '_user', None)

    def _set_user(self, user):
        self._user = user

    user = property(_get_user, _set_user)

    def clean(self):
        if any(self.errors):
            return
        if not self.user:
            self._errors['confirm'] = [_("No user has been set")]
            raise forms.ValidationError()
        Team.objects.convert_from_user(self.user)
        return super(UserConvertToTeamForm, self).clean()

map = get_config_value('FORK_EXTERNAL_MAP')
fork_map = dict((key, str2obj(val)) for key, val in map.items())
choices = tuple((key, key) for key in fork_map)


class ExternalForkSourcesForm(forms.Form):
    source = forms.ChoiceField(choices=choices)


class ExternalForkWizard(FormWizard):

    def done(self, request, form_list):
        form = form_list[1]
        form.request = request
        try:
            fork = form.fork()
        except ProjectorError, err:
            msg = _("Error occured while trying to fork %s" % form.get_url())
            messages.error(request, msg)
            logging.error(str(err))
            return redirect('projector_dashboard')
        else:
            return redirect(fork.get_absolute_url())

    def process_step(self, request, form, step):
        form.is_valid()
        if step == 0:
            source = form.cleaned_data['source']
            self.form_list[1] = fork_map[source]
            # TODO: Setting request here is kind of *magic*
            self.form_list[1].request = request
        elif step == 1:
            if not isinstance(form, BaseExternalForkForm):
                raise ProjectorError(
                        "Final fork wizard form must be subclass of "
                        "projector.forks.base.BaseForkForm class")
        return super(ExternalForkWizard, self).process_step(request, form, step)

    def get_template(self, step):
        return 'projector/accounts/dashboard-external-fork.html'


class DashboardAddMemberForm(forms.Form):
    user = UserByNameField()

    def __init__(self, group, *args, **kwargs):
        self.group = group
        super(DashboardAddMemberForm, self).__init__(*args, **kwargs)

    def clean_user(self):
        """
        Raises ``ValidationError`` if user is already member of given group or
        user does not exist.
        """
        user = self.cleaned_data['user']
        if user:
            try:
                user.groups.get(name=self.group.name)
            except Group.DoesNotExist:
                pass
            else:
                raise forms.ValidationError(_("User %(user)s is already a "
                    "member of %(group)s" %
                    {'user': user, 'group': self.group}))
        return user

    def save(self, commit=True):
        user = self.cleaned_data['user']
        if commit:
            user.groups.add(self.group)
        return user

